from typing import List
from typing import TypedDict

from testlodge._types import DateTimeStr
from testlodge._types import Pagination


class PlanJSON(TypedDict):

    id: int
    name: str
    test_plan_identifier: str
    project_id: int
    created_at: DateTimeStr
    updated_at: DateTimeStr


class PlanListJSON(TypedDict):

    pagination: Pagination
    steps: List[PlanJSON]



from typing import Dict
from typing import Optional

from furl import Path as UrlPath
from furl.furl import furl as Url
from requests.models import Response
from testlodge.api.base import BaseAPI
from testlodge.models.project import ProjectJSON
from testlodge.models.project import ProjectListJSON


class ProjectAPI(BaseAPI):
    """API for projects.

    Endpoints
    ---------
    * List
    * Show
    * Create
    * Update
    * Delete
    """

    name: str = 'project'

    def _list(
        self,
        page: int = 1,
    ) -> ProjectListJSON:
        """Paginated list of all projects for a user.

        Parameters
        ----------
        page: int, default=1
            Default: 1
            Page to return.
        """

        method: str = 'GET'
        url: Url = self.client.base_url / UrlPath('/projects.json')
        if page != 1:
            params = {'page': page}
        else:
            params = {}

        response: Response = self.client._request(
            method=method, url=url, params=params
        )
        project_list: ProjectListJSON = response.json()

        return project_list

    def _show(
        self,
        project_id: int,
    ) -> ProjectJSON:
        """Get the details for a project.

        Parameters
        ----------
        project_id: Identifier
            The ID of the project.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}.json'
        )

        response: Response = self.client._request(
            method=method,
            url=url,
        )
        project_json: ProjectJSON = response.json()

        return project_json

    def _create(
        self,
        name: str,
        description: Optional[str] = None,
        issue_tracker_credential_id: Optional[int] = None,
        issue_tracker_project_id: Optional[str] = None,
    ) -> ProjectJSON:
        """Create a project.

        Parameters
        ----------
        name: str
            Name of the project.
        description: str, optional
            Project description.
        issue_tracker_credential_id: int, optional
            ID for an integrated issue tracker.
        issue_tracker_project_id: str, optional
            Project ID associated with integrated issue tracker.
        """

        method = 'POST'
        url: Url = self.client.base_url / UrlPath('/projects.json')

        data: Dict = dict(name=name)
        if description is not None:
            data['description'] = description
        if issue_tracker_credential_id is not None:
            data['issue_tracker_credential_id'] = str(
                issue_tracker_credential_id
            )
        if issue_tracker_project_id is not None:
            data['issue_tracker_project_id'] = issue_tracker_project_id

        response: Response = self.client._request(
            method=method, url=url, json=data
        )
        project_json: ProjectJSON = response.json()

        return project_json

    def _update(
        self,
        project_id: int,
        project: ProjectJSON,
    ) -> ProjectJSON:
        """Update a project.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        project: ProjectJSON
            name: str
                Name of the project.
            description: str
                Project description.
            issue_tracker_credential_id: int
                ID for an integrated issue tracker.
            issue_tracker_project_id: str
                Project ID associated with integrated issue tracker.
        """

        method = 'PATCH'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}.json'
        )

        response: Response = self.client._request(
            method=method,
            url=url,
            json=project,
        )
        project_json: ProjectJSON = response.json()

        return project_json

    def _delete(
        self,
        project_id: int,
    ) -> None:
        """Delete a project.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        """

        method = 'DELETE'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}.json'
        )

        response: Response = self.client._request(method=method, url=url)

        status_code: int = response.status_code
        if status_code != 204:
            print(f'Unexpected response code: {status_code}')

        return None



from typing import List
from typing import Optional
from typing import TypedDict

from testlodge._types import DateTimeStr
from testlodge._types import Pagination
from testlodge.models.custom_field import CustomFieldJSON
from testlodge.models.user import UserJSON


class PathJSON(TypedDict):
    name: str
    geometry: Optional[str]
    url: str


class PathsJSON(TypedDict):
    original: PathJSON
    thumbnail: PathJSON


class UploadJSON(TypedDict):

    id: int
    file_name: str
    # MIME Type
    content_type: str
    paths: PathsJSON


class RequirementJSON(TypedDict):

    id: int
    requirement_number: str
    title: str
    description: str
    position: int
    requirement_document_id: int
    last_saved_by_id: int
    last_saved_by: UserJSON
    requirement_uploads: List[UploadJSON]
    custom_fields: List[CustomFieldJSON]
    created_at: DateTimeStr
    updated_at: DateTimeStr


class RequirementListJSON(TypedDict):

    pagination: Pagination
    requirements: List[RequirementJSON]



from typing import List
from typing import Optional

from furl import Path as UrlPath
from furl.furl import furl as Url
from requests.models import Response
from testlodge.api.base import BaseAPI
from testlodge.models.requirement import RequirementJSON
from testlodge.models.requirement import RequirementListJSON


class RequirementAPI(BaseAPI):
    """API for test suites.

    Endpoints
    ---------
    * List
    * Show
    * Create
    * Update
    * Delete
    """

    name: str = 'requirement'

    def _list(
        self,
        project_id: int,
        requirement_document_id: int,
        page: int = 1,
    ) -> RequirementListJSON:
        """Paginated list of all requirements in a requirement document.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        requirement_document_id: int
            The ID of the requirement document.
        page: int, default=1
            Default: 1
            The number of the page to return.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/requirement_documents/{requirement_document_id}'
            '/requirements.json'
        )
        params: dict = {}
        if page != 1:
            params['page'] = page

        response: Response = self.client._request(
            method=method, url=url, params=params
        )
        requirement_list: RequirementListJSON = response.json()

        return requirement_list

    def _show(
        self,
        project_id: int,
        requirement_document_id: int,
        requirement_id: int,
        include: Optional[List[str]] = None,
    ) -> RequirementJSON:
        """Get the details for a requirement.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        requirement_document_id: int
            The ID of the requirement document.
        requirement_id: int
            The ID of the requirement.

        include: dict[str], optional
            An array of strings, representing the additional options to include
            in the response.

            requirement_uploads: str
                Any file that has been uploaded and associated with the
                requirement (urls in the response will only be valid
                for ~30 seconds).
            steps: list[str]
                Any steps that have been associated to the case.
        """

        method = 'GET'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/requirement_documents/{requirement_document_id}'
            f'/requirements/{requirement_id}.json'
        )

        params = {}
        if include is not None:
            raise NotImplementedError('Not implemented yet.')
        else:
            ...

        response: Response = self.client._request(
            method=method,
            url=url,
            params=params,
        )
        suite_json: RequirementJSON = response.json()

        return suite_json

    def _create(
        self,
        project_id: int,
        requirement_document_id: int,
        requirement: RequirementJSON,
    ) -> RequirementJSON:
        """Create a requirement.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        requirement_document_id: int
            The ID of the requirement document.
        requirement: RequirementJSON

            title: str
                Title of the requirement.
            description: str
                Description of the requirement.
            custom_fields: List[CustomFieldJSON]
        """

        method = 'POST'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/requirement_documents/{requirement_document_id}'
            '/requirements.json'
        )

        data = dict(requirement=requirement)

        response: Response = self.client._request(
            method=method, url=url, json=data
        )
        requirement_json: RequirementJSON = response.json()

        return requirement_json

    def _update(
        self,
        project_id: int,
        requirement_document_id: int,
        requirement_id: int,
        requirement: RequirementJSON,
    ) -> RequirementJSON:
        """Update a requirement.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        requirement_document_id: int
            The ID of the requirement document.
        requirement_id: int
            The ID of the requirement.
        requirement: RequirementJSON

            title: str
                Title of the requirement.
            description: str
                Description of the requirement.
            custom_fields: List[CustomFieldJSON]
        """

        method = 'PATCH'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/requirement_documents/{requirement_document_id}'
            f'/requirements/{requirement_id}.json'
        )
        data = dict(requirement=requirement)

        response: Response = self.client._request(
            method=method,
            url=url,
            json=data,
        )
        requirement_json: RequirementJSON = response.json()

        return requirement_json

    def _delete(
        self,
        project_id: int,
        requirement_document_id: int,
        requirement_id: int,
    ) -> None:
        """Delete a requirement.

        Parameters
        ----------
        project_id: int
            The ID of the project.
        requirement_document_id: int
            The ID of the requirement document.
        requirement_id: int
            The ID of the requirement.
        """

        method = 'DELETE'
        url: Url = self.client.base_url / UrlPath(
            f'/projects/{project_id}'
            f'/requirement_documents/{requirement_document_id}'
            f'/requirements/{requirement_id}.json'
        )

        response: Response = self.client._request(method=method, url=url)

        status_code: int = response.status_code
        if status_code != 204:
            print(f'Unexpected response code: {status_code}')

        return None
